/*
 * This souce code is public domain. Enjoy!
 * http://hildstrom.com/projects/buellstator
 *
 * Last update: 201204191144
 *
 * 4-sensor data logger for Arduino Uno
 * stator thermocouple, regulator thermistor, air thermistor, battery voltage
 *
 * PINS
 * D2 -> MAX6675-1 DO
 * D3 -> MAX6675-1 CS
 * D4 -> MAX6675-1 CLK
 * +5V -> 10K thermistor -> A0 -> 10K resistor -> GND
 * +5V -> 10K thermistor -> A1 -> 10K resistor -> GND
 * +15V -> 20K resistor -> A2 -> 10K resistor -> GND
 *
 * EEPROM layout
 * [0-1019] interleaved 8-bit scan data st,rt,at,bv,st,rt,at,bv...
 * [1020-1021] not used
 * [1022] init byte
 * [1023] next scan number 1020 bytes / 4 sensors = 255 scans for 0-254
 *
 * The logger will save scans and revolve to overwrite old scans.
 * Scan 254 occupies addresses 254*4=1016 to 1019 in the EEPROM.
 * The very next scan should be 0. The variables should make it easy to
 * adapt to other EEPROM sizes or number of sensors.
 * The log_delay >= display_delay.
 */

//includes
#include <EEPROM.h> //for EEPROM I/O
#include "max6675.h" //for MAX6675 thermocouple IC
#include <math.h>  //for thermistor temperature

//global variables
//user-defined
const byte use_eeprom = 1;
const unsigned short num_sensors = 4;
const char* sensor_names[num_sensors]={
  "stator_thermocouple",
  "regulator_thermistor",
  "air_thermistor",
  "battery_voltage"};
const unsigned short first_address = 0;
const unsigned short scan_address = 1023;
const unsigned long display_delay = 1000;
const unsigned long log_delay = 60000;
const byte stator_tc_do_pin = 2;
const byte stator_tc_cs_pin = 3;
const byte stator_tc_clk_pin = 4;
const byte regulator_temp_pin = 0;
const byte air_temp_pin = 1;
const byte batt_volt_pin = 2;
//internal
const unsigned short init_address = scan_address-1;
const unsigned short num_scans = (scan_address-2-first_address+1)/num_sensors;
unsigned short scan = 0;
unsigned short sensor = 0;
unsigned short address = 0;
unsigned long current_milliseconds = 0;
unsigned long previous_display_milliseconds = 0;
unsigned long previous_log_milliseconds = 0;
MAX6675 stator_tc(stator_tc_clk_pin,stator_tc_cs_pin,stator_tc_do_pin);
byte sensor_values[num_sensors]={
  0,
  0,
  0,
  0};

//function to convert a 10k thermistor adc value to temperature
//from http://arduino.cc/playground/ComponentLib/Thermistor2
double thermistor(int RawADC) {
 double Temp;
 Temp = log(((10240000/RawADC) - 10000));
 Temp = 1 / (0.001129148 + (0.000234125 + (0.0000000876741 * Temp * Temp ))* Temp );
 Temp = Temp - 273.15;            // Convert Kelvin to Celcius
 //Temp = (Temp * 9.0)/ 5.0 + 32.0; // Convert Celcius to Fahrenheit
 return Temp;
}

//function to convert double to byte
byte double_to_byte(double d){
  if(d>255){
    return 255;
  }
  else if(d<0){
    return 0;
  }
  else{
    return (byte)d;
  }
}

//setup function
void setup(){
  //setup serial port
  Serial.begin(9600);
  
  //setup i/o here
  
  //only dump the eeprom if we are using the eeprom
  if(use_eeprom){
    //wait 5 seconds
    delay(5000);
    Serial.println("LOGDATASTART");
    
    //check if the eeprom has been initialized before
    if(EEPROM.read(init_address) != 27){
      for(address=first_address; address<=scan_address; address++){
        EEPROM.write(address,0);
      }
      EEPROM.write(init_address,27);
    }
    
    //dump the eeprom to the computer over serial/usb
    Serial.print("scan");
    Serial.print("\t");
    for(sensor=0; sensor<num_sensors; sensor++){
      Serial.print(sensor_names[sensor]);
      if(sensor != num_sensors-1){
        Serial.print("\t");
      }
    }
    Serial.println();
    for(scan=0; scan<num_scans; scan++){
      address=scan*num_sensors;
      Serial.print(scan,DEC);
      Serial.print("\t");
      for(sensor=0; sensor<num_sensors; sensor++){
        Serial.print(EEPROM.read(address+sensor),DEC);
        if(sensor != num_sensors-1){
          Serial.print("\t");
        }
      }
      Serial.println();
    }
    Serial.println("LOGDATAEND");
    Serial.println();
    Serial.println();
    Serial.println();
    Serial.println();
    Serial.println();
    
    //read the next scan number stored in the EEPROM
    scan = EEPROM.read(scan_address);
    
    //wait 5 seconds
    delay(5000);
  }
  
  //setup start time
  current_milliseconds = millis();
  previous_display_milliseconds = current_milliseconds;
  previous_log_milliseconds = current_milliseconds;
  
  //serial/usb outupt header
  Serial.print("scan");
  Serial.print("\t");
  for(sensor=0; sensor<num_sensors; sensor++){
    Serial.print(sensor_names[sensor]);
    if(sensor != num_sensors-1){
      Serial.print("\t");
    }
  }
  Serial.println();
}

//loop function
void loop(){
  //get elapsed time
  current_milliseconds = millis();
  
  //read once per display delay
  if(current_milliseconds - previous_display_milliseconds > display_delay){
    previous_display_milliseconds = current_milliseconds;
    
    //perform measurements
    sensor_values[0] = double_to_byte(stator_tc.readCelsius());
    sensor_values[1] = double_to_byte(thermistor(analogRead(regulator_temp_pin)));
    sensor_values[2] = double_to_byte(thermistor(analogRead(air_temp_pin)));
    sensor_values[3] = analogRead(batt_volt_pin)/4;//converting 10-bit to 8-bit
    
    //output results
    Serial.print(scan,DEC);
    Serial.print("\t");
    for(sensor=0; sensor < num_sensors; sensor++){
      Serial.print(sensor_values[sensor],DEC);
      if(sensor != num_sensors-1){
        Serial.print("\t");
      }
    }
    Serial.println();
  }
  
  //log once per log delay
  if(current_milliseconds - previous_log_milliseconds > log_delay && use_eeprom){
    previous_log_milliseconds = current_milliseconds;
    
    //save results
    address=scan*num_sensors;
    for(sensor=0; sensor < num_sensors; sensor++){
      EEPROM.write(address+sensor,sensor_values[sensor]);
    }
    
    //revolve back to beginning of eeprom address space when necessary
    if(scan >= num_scans-1){
      scan=0;
    }
    else{
      scan++;
    }
    EEPROM.write(scan_address,scan);
  }
  
}

